//////////////////////////////////////////////////////////////////////
//
//
//		File :					"PhoenixCDPlayer.cpp"
//
//
//		Related header file :	"PhoenixCDPlayer.h"
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				CD PLayer.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Included headers
//
//
//////////////////////////////////////////////////////////////////////
/*  CDPLayer part of library  */
#include	"PhoenixCDPlayer.h"

//////////////////////////////////////////////////////////////////////
//
//
//		Global variables
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Functions
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CDPlayerError"
//
//
//		Input :					ULONG ulError - Error that happen
//								WORD Flags	  - Flags
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				Error handler for CDPlayer fucntions
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
void CDPlayerError (ULONG ulError, WORD wFlags)

{


	char		ErrorMsg[256];
	ZeroMemory(ErrorMsg, 256);

	switch (ulError)

	{

	case ERRCDPLAYER_OPEN:

		strcpy (ErrorMsg, "CCDPlayer() failed - Error 2001");

		if (wFlags & ERRCDPLAYER_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRCDPLAYER_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

	case ERRCDPLAYER_SETTIMEFORMAT:

		strcpy (ErrorMsg, "CCDPlayer() failed - Error 2002");

		if (wFlags & ERRCDPLAYER_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRCDPLAYER_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

	case ERRCDPLAYER_CLOSE:

		strcpy (ErrorMsg, "~CCDPlayer() failed - Error 2003");

		if (wFlags & ERRCDPLAYER_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRCDPLAYER_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRCDPLAYER_PLAYTRACK:

		strcpy (ErrorMsg, "PlayTrack() failed - Error 2004");

		if (wFlags & ERRCDPLAYER_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRCDPLAYER_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

	case ERRCDPLAYER_STOPTRACK:

		strcpy (ErrorMsg, "StopTrack() failed - Error 2005");

		if (wFlags & ERRCDPLAYER_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRCDPLAYER_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

	default:

		break;

	}

	if (wFlags & ERRCDPLAYER_QUIT)

	{

		PostQuitMessage (ulError);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CCDPlayer::CCDPlayer()"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				Creates and initializes the CDPlayer.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CCDPlayer::CCDPlayer()

{
    MCI_OPEN_PARMS			mciOpenParms;
    MCI_SET_PARMS			mciSetParms;

    mciOpenParms.lpstrDeviceType = "cdaudio";

    if (mciSendCommand(NULL, MCI_OPEN, MCI_OPEN_TYPE, (DWORD)(LPVOID) 
		&mciOpenParms))
    
	{

		CDPlayerError (ERRCDPLAYER_OPEN, ERRCDPLAYER_FILE);

    }

    wDeviceID = mciOpenParms.wDeviceID;

    mciSetParms.dwTimeFormat = MCI_FORMAT_TMSF;

    if (mciSendCommand(wDeviceID, MCI_SET, MCI_SET_TIME_FORMAT, 
					   (DWORD)(LPVOID) &mciSetParms))
    
	{

		CDPlayerError (ERRCDPLAYER_SETTIMEFORMAT, ERRCDPLAYER_FILE);
    
    } 
        
}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CCDPlayer::~CCDPlayer()"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				Destroys the CDPlayer.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CCDPlayer::~CCDPlayer()

{

	if (mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL))

	{

		CDPlayerError (ERRCDPLAYER_CLOSE, ERRCDPLAYER_FILE);
    
    } 
        
}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CCDPlayer::PlayTrack()"
//
//
//		Input :					BYTE byTrack - Track to play
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				Plays a specified track.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CCDPlayer::PlayTrack(BYTE byTrack)

{

    MCI_PLAY_PARMS mciPlayParms;

    mciPlayParms.dwFrom = 0L;
    mciPlayParms.dwTo = 0L;
    mciPlayParms.dwFrom = MCI_MAKE_TMSF(byTrack, 0, 0, 0);
    mciPlayParms.dwTo = MCI_MAKE_TMSF(byTrack + 1, 0, 0, 0);

	if (mciSendCommand(wDeviceID, MCI_PLAY, MCI_FROM | MCI_TO | MCI_NOTIFY, 
					  (DWORD)(LPVOID) &mciPlayParms))

	{

		CDPlayerError (ERRCDPLAYER_PLAYTRACK, ERRCDPLAYER_FILE);
    
    } 

	return 0;

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CCDPlayer::StopTrack()"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			08 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			08 - 02 - 2000
//
//
//		Purpose :				Stops a specified track.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CCDPlayer::StopTrack()

{

    MCI_PLAY_PARMS mciPlayParms;

	if (mciSendCommand(wDeviceID, MCI_STOP, MCI_NOTIFY, 
					  (DWORD)(LPVOID) &mciPlayParms))

	{

		CDPlayerError (ERRCDPLAYER_STOPTRACK, ERRCDPLAYER_FILE);
    
    } 

	return 0;

}
