//////////////////////////////////////////////////////////////////////
//
//
//		File :					"PhoenixGraphics.cpp"
//
//
//		Related header file :	"PhoenixGraphics.h"
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			02 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				DirectDraw object implementation.
//								All primitive graphics functions are
//								declared in this file. 
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Included headers
//
//
//////////////////////////////////////////////////////////////////////
/*  Graphics part of library  */
#include	"PhoenixGraphics.h"

//////////////////////////////////////////////////////////////////////
//
//
//		Global variables
//
//
//////////////////////////////////////////////////////////////////////
LPDIRECTDRAW7					g_lpDirectDraw7;


//////////////////////////////////////////////////////////////////////
//
//
//		Functions
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"GraphicError"
//
//
//		Input :					ULONG ulError - Error that happen
//								WORD Flags	  - Flags
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Error handler for graphic fucntions
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
void GraphicError (ULONG ulError, WORD wFlags)

{

	char		ErrorMsg[256];
	ZeroMemory(ErrorMsg,256);

	switch (ulError)

	{

	case ERRGRAPHICS_CREATE_SURFACE:

		strcpy (ErrorMsg, "CreateSurface() failed - Error 0001");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_INIT_COLOR	:

		strcpy (ErrorMsg, "InitColor() failed - Error: 0002");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_LOCK_SURFACE:

		strcpy (ErrorMsg, "LockSurface() failed. - Error 0003");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_UNLOCK_SURFACE:

		strcpy (ErrorMsg, "UnlockSurface() failed - Error 0004");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_CHECK_LOST:

		strcpy (ErrorMsg, "CheckLost() failed - Error 0005");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_RESTORE:

		strcpy (ErrorMsg, "CheckLost() failed - Error 0006");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_BLIT:

		strcpy (ErrorMsg, "Blit() failed - Error 0007");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_BLITEX:

		strcpy (ErrorMsg, "BlitEx() failed - Error 0008");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_CLEAR_SURFACE:

		strcpy (ErrorMsg, "ClearSurface() failed - Error 0009");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_RELEASE_SURFACE:

		strcpy (ErrorMsg, "~CSurface() failed - Error 0010");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_DRAW_PIXEL:

		strcpy (ErrorMsg, "DrawPixel() failed - Error 0011");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_GET_INFO:

		strcpy (ErrorMsg, "GetInfo() failed - Error 0012");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_DD_SET_COOPERATIVE:

		strcpy (ErrorMsg, "CScreen() failed - Error 0013");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_DD_CREATE:

		strcpy (ErrorMsg, "CScreen() failed - Error 0014");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_FLIP:

		strcpy (ErrorMsg, "Flip() failed - Error 0015");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_SET_DISPLAY_MODE:

		strcpy (ErrorMsg, "SetDisplayMode() failed - Error 0016");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_CREATE_PRIMARY:

		strcpy (ErrorMsg, "SetDisplayMode() failed - Error 0017");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_CREATE_SECONDARY:

		strcpy (ErrorMsg, "SetDisplayMode() failed - Error 0018");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	case ERRGRAPHICS_RELEASE_SCREEN:

		strcpy (ErrorMsg, "~CScreen() failed - Error 0019");

		if (wFlags & ERRGRAPHICS_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRGRAPHICS_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;

	}

	if (wFlags & ERRGRAPHICS_QUIT)

	{

		PostQuitMessage (ulError);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::CSurface"
//
//
//		Input :					WORD wWidth  - Width of surface
//								WORD wHeight - Height of surface
//								BYTE byFlags - Video or system memory
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Creates and initializes the CSurface
//								object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::CSurface (WORD wWidth, WORD wHeight, BYTE byFlags)

{

	/*  Needed variables  */
	DDSURFACEDESC2		ddsd;
	HRESULT				hResult;

	/*  Clears and set struct size  */
    ZeroMemory (&ddsd, sizeof(ddsd));
	ddsd.dwSize			= sizeof (ddsd);

	/*  Structure parameters  */
	ddsd.dwFlags		= DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH; 

	if (byFlags & SURFACE_VIDEO)

	{

		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY; 

	}

	else if (byFlags & SURFACE_SYSTEM)

	{

		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY; 
		bSystemSurface = true;

	}

	else 

	{

		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN; 

	}

	ddsd.dwHeight		= wHeight; 
	ddsd.dwWidth		= wWidth; 

	/*  Creates surface  */
	hResult = g_lpDirectDraw7->CreateSurface (&ddsd, &lpSurface, NULL);

	if (FAILED (hResult))

	{

		GraphicError (ERRGRAPHICS_CREATE_SURFACE, ERRGRAPHICS_QUIT | ERRGRAPHICS_FILE);

	}

	else

	{

		wRedShift	 = 0;
		wGreenShift	 = 0;
		wBlueShift	 = 0;
		wWidth		 = 0;
		wHeight		 = 0;
		wPitch		 = 0;
		pwBuffer	 = NULL;
		bLocked		 = false;
		bSystemSurface = false;

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::CSurface"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Creates and initializes the CSurface
//								object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::CSurface ()

{

	bLocked = false;
	bSystemSurface = false;

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::~CSurface"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Releases CSurface object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::~CSurface ()

{

	HRESULT		hResult;

	hResult = lpSurface->Release();

	if (FAILED(hResult))

		{

			GraphicError (ERRGRAPHICS_RELEASE_SURFACE, ERRGRAPHICS_FILE);

		}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::LockSurface"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Lock the surface for direct 
//								manipulation.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::LockSurface ()

{

	HRESULT				hResult;

	if (bSystemSurface)

	{

		DDSURFACEDESC2		ddsd;

		ZeroMemory(&ddsd, sizeof(ddsd));
		ddsd.dwSize=sizeof(DDSURFACEDESC2);

		hResult = lpSurface->GetSurfaceDesc(&ddsd);

		pwBuffer = (WORD*)malloc (ddsd.dwWidth * ddsd.dwHeight);

		pwBuffer = (WORD*)ddsd.lpSurface;

		if (FAILED(hResult))

		{

			GraphicError (ERRGRAPHICS_LOCK_SURFACE, ERRGRAPHICS_FILE);

		}

		else

		{

			bLocked = true;
			wPitch = ddsd.lPitch / 2;

		}
		
		return (0);

	}

	if (!bLocked)

	{

		DDSURFACEDESC2		ddsd;

		ZeroMemory(&ddsd, sizeof(ddsd));
		ddsd.dwSize=sizeof(DDSURFACEDESC2);

		hResult = lpSurface->Lock(NULL, &ddsd, NULL, NULL);

		pwBuffer = (WORD*)malloc (ddsd.dwWidth * ddsd.dwHeight);

		pwBuffer = (WORD*)ddsd.lpSurface;

		if (FAILED(hResult))

		{

			GraphicError (ERRGRAPHICS_LOCK_SURFACE, ERRGRAPHICS_FILE);

		}

		else

		{

			bLocked = true;
			wPitch = ddsd.lPitch / 2;

		}

	}

	else

	{

		GraphicError (ERRGRAPHICS_LOCK_SURFACE, ERRGRAPHICS_FILE);

	}	

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::UnlockSurface"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Unlock the surface to prevent
//								direct manipulation.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::UnlockSurface ()

{

	HRESULT hResult;

	if (bSystemSurface)

	{

		bLocked	= false;
	
		return (0);

	}

	hResult = lpSurface->Unlock(NULL);

	if (FAILED(hResult))

	{

		GraphicError (ERRGRAPHICS_UNLOCK_SURFACE, ERRGRAPHICS_FILE);

	}

	if (!FAILED (hResult))

	{

		bLocked = false;

	}

	return (0);

}

//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::SetColorKey"
//
//
//		Input :					BYTE byType - Source or destination
//								WORD wRed	- Red component
//								WORD wGreen	- Green component
//								WORD wBlue	- Blue componet
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Set the source/destination key of a
//								surface.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::SetColorKey (BYTE byType, WORD wRed, WORD wGreen, WORD wBlue)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::Blit"
//
//
//		Input :					RECT rcArea				- Area to blit
//								CSurface CSourceSurface - Source 
//														  Surface
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Blits a surface to this one.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::Blit (RECT rcArea, RECT rcAreaDst, CSurface CSourceSurface)

{

	HRESULT hResult;

	if (!bLocked)

	{

		hResult = lpSurface->Blt(&rcArea , 
								   CSourceSurface.lpSurface, 
								   &rcAreaDst, DDBLT_WAIT, NULL);

		if (FAILED(hResult))
			
		{

			GraphicError (ERRGRAPHICS_BLIT, ERRGRAPHICS_FILE);

		}

	}

	else

	{

		GraphicError (ERRGRAPHICS_LOCK_SURFACE, ERRGRAPHICS_FILE);

	}
	
	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::BlitEx"
//
//
//		Input :					RECT rcArea				- Area to blit
//								CSurface CSourceSurface - Source 
//														  Surface
//								DWORD					- dwFlags
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Blits a surface to this using special
//								flags.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::BlitEx (RECT rcArea, CSurface CSourceSurface, DWORD dwFlags)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::ClearSurface"
//
//
//		Input :					WORD wRed	- Red component
//								WORD wGreen	- Green component
//								WORD wBlue	- Blue component
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Clears the surface to a certain
//								color.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::ClearSurface (WORD wRed, WORD wGreen, WORD wBlue)

{

	DDBLTFX ddbltfx; 
	HRESULT	hResult;

	ZeroMemory(&ddbltfx, sizeof(ddbltfx));
	ddbltfx.dwSize=sizeof(DDBLTFX);

	ddbltfx.dwFillColor = (WORD) (wRed << wRedShift | 
								  wGreen << wGreenShift | 
								  wBlue << wBlueShift); 

	hResult = lpSurface->Blt(NULL,NULL,NULL, DDBLT_COLORFILL | 
							   DDBLT_WAIT, &ddbltfx);  

	if (FAILED(hResult))

	{

		GraphicError (ERRGRAPHICS_CLEAR_SURFACE, ERRGRAPHICS_FILE);

	}

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::DrawPixel"
//
//
//		Input :					WORD wX		- X position
//								WORD wY		- Y position
//								WORD wRed	- Red component
//								WORD wGreen	- Green component
//								WORD wBlue	- Blue component
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Draw a pixel in the surface.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::DrawPixel (WORD wX, WORD wY, WORD wRed, WORD wGreen, WORD wBlue)

{

	if (bLocked)

	{

		pwBuffer [(wX + wY * wPitch)]= (WORD)(wRed << wRedShift | 
												wGreen << wGreenShift | 
												wBlue << wBlueShift); 

	}

	else

	{

		GraphicError (ERRGRAPHICS_DRAW_PIXEL, ERRGRAPHICS_FILE);

	}
	
	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::DrawAlphaPixel"
//
//
//		Input :					WORD wX		 - X position
//								WORD wY		 - Y position
//								WORD wRed	 - Red component
//								WORD wGreen	 - Green component
//								WORD wBlue	 - Blue component
//								BYTE byAlpha - Alpha value 
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Draw a pixel in the surface using 
//								alphablending.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::DrawAlphaPixel (WORD wX, WORD wY, WORD wRed, WORD wGreen, WORD wBlue, 
				BYTE byAlpha)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::DrawLine"
//
//
//		Input :					RECT rcOutline - Outline of line
//								WORD wRed	   - Red component
//								WORD wGreen	   - Green component
//								WORD wBlue	   - Blue component
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Draws a line in the surface.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::DrawLine (RECT rcOutline, WORD wRed, WORD wGreen, WORD wBlue)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::DrawAlphaLine"
//
//
//		Input :					RECT rcOutline - Outline of line
//								WORD wRed	   - Red component
//								WORD wGreen	   - Green component
//								WORD wBlue	   - Blue component
//								BYTE byAlpha   - Alpha value
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Draws a line in the surface using
//								alphablending.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::DrawAlphaLine (RECT rcOutline, WORD wRed, WORD wGreen, WORD wBlue, 
			   BYTE byAlpha)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::LoadImage"
//
//
//		Input :					LPSTR lpstrFileName - Source file
//								DWORD dwType		- Type of file
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Loads a bitmapped image in the surface. 
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::LoadImage (LPSTR lpstrFileName, DWORD dwType)

{

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::CheckLost"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Checks if a surface is lost and 
//								restores it if it is.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::CheckLost ()

{

	HRESULT hResult;

	hResult = lpSurface->IsLost();

	if (hResult == DDERR_SURFACELOST)

	{

		hResult = lpSurface->Restore();

		if (FAILED (hResult))

		{
			
			GraphicError (ERRGRAPHICS_RESTORE, ERRGRAPHICS_FILE);

		}

	}

	else 

	{

		if (FAILED (hResult))

		{
			
			GraphicError (ERRGRAPHICS_CHECK_LOST, ERRGRAPHICS_FILE);

		}

	}

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::InitColor"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Initializes the shift switching values.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::InitColor()

{

	DDPIXELFORMAT		ddpf;
	HRESULT				hResult;

	wRedShift=0;
	wGreenShift=0;
	wBlueShift=0;

	ZeroMemory(&ddpf, sizeof(ddpf));
	ddpf.dwSize=sizeof(DDPIXELFORMAT);

	hResult = lpSurface->GetPixelFormat(&ddpf);

	if (FAILED(hResult))

	{

		GraphicError(ERRGRAPHICS_INIT_COLOR, ERRGRAPHICS_FILE);

	}


	if (!FAILED (hResult)) 

	{
		
	    while(!(ddpf.dwRBitMask & 0x01L))

		{

	    	ddpf.dwRBitMask >>= 1;
		    wRedShift++;

	    }
		   
		while(!(ddpf.dwGBitMask & 0x01L))

	    {

			ddpf.dwGBitMask >>= 1;
			wGreenShift++;

		}
		
		while(!(ddpf.dwBBitMask & 0x01L))

	    {

			ddpf.dwBBitMask >>= 1;
			wBlueShift++;

	    }

	}

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CSurface::GetInfo"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Get all info about the surface.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CSurface::GetInfo ()

{

	HRESULT				hResult;

	ZeroMemory(&sSurface, sizeof(sSurface));
	sSurface.dwSize=sizeof(DDSURFACEDESC2);

	hResult = lpSurface->GetSurfaceDesc(&sSurface);

	if (FAILED(hResult))

	{

		GraphicError (ERRGRAPHICS_GET_INFO, ERRGRAPHICS_FILE);

	}

	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CScreen::CScreen"
//
//
//		Input :					HWND hWnd - Handle to parent window
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			05 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				Creates the CScreen object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CScreen::CScreen(HWND hWnd)

{

	HRESULT			hResult;
	
	hResult=DirectDrawCreateEx(NULL, (VOID**) &g_lpDirectDraw7, 
							   IID_IDirectDraw7, NULL);

	if (!FAILED(hResult))

	{

		hResult=g_lpDirectDraw7->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | 
									 				 DDSCL_FULLSCREEN); 

		if (FAILED(hResult))

		{

			GraphicError (ERRGRAPHICS_DD_SET_COOPERATIVE, ERRGRAPHICS_FILE);

		}

	}

	else

	{

		GraphicError (ERRGRAPHICS_DD_CREATE, ERRGRAPHICS_FILE);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CScreen::~CScreen"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			05 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				Destructs the CScreen object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CScreen::~CScreen()

{

	HRESULT		hResult;

	hResult = g_lpDirectDraw7->Release();

	if (FAILED(hResult))

		{

			GraphicError (ERRGRAPHICS_RELEASE_SCREEN, ERRGRAPHICS_FILE);

		}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CScreen::SetDisplayMode"
//
//
//		Input :					WORD wWidth  - Width of screen
//								WORD wHeight - Height of screen
//								WORD wBpp	 - Bits per pixel
//								BYTE byFlags - Flags
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			05 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				Sets the display mode.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CScreen::SetDisplayMode (WORD wWidth, WORD wHeight, WORD wBpp, 
						 BYTE byFlags)

{

	HRESULT				hResult;
	DDSURFACEDESC2		ddsd;
    DDSCAPS2            ddscaps;

	hResult = g_lpDirectDraw7->SetDisplayMode(wWidth, wHeight, wBpp, 0, 0); 

	if (!FAILED(hResult))

	{

	    ZeroMemory(&ddsd, sizeof(ddsd));
	    ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT; 
	    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP |
                          DDSCAPS_COMPLEX;
		ddsd.dwBackBufferCount = 1;

		switch (byFlags)

		{

		case (SURFACE_PRIMARY):

			hResult = g_lpDirectDraw7->CreateSurface(&ddsd, &CPrimary.lpSurface, NULL); 
		
			if (FAILED(hResult))

			{

				GraphicError (ERRGRAPHICS_CREATE_PRIMARY, ERRGRAPHICS_FILE);

			}

			break;

		case (SURFACE_PRIMARY_SECONDARY):

			hResult = g_lpDirectDraw7->CreateSurface(&ddsd, &CPrimary.lpSurface, NULL); 

			if (!FAILED(hResult))

			{

				ZeroMemory(&ddscaps, sizeof(ddscaps));
				ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
				hResult = CPrimary.lpSurface->GetAttachedSurface(&ddscaps, &CSecondary.lpSurface); 

				if (FAILED(hResult))

				{

					GraphicError (ERRGRAPHICS_CREATE_SECONDARY, ERRGRAPHICS_FILE);

				}

			}

			if (FAILED(hResult))

			{

				GraphicError (ERRGRAPHICS_CREATE_PRIMARY, ERRGRAPHICS_FILE);

			}	
		
			return (0);

			break;

		default:

			hResult = g_lpDirectDraw7->CreateSurface(&ddsd, &CPrimary.lpSurface, NULL); 

			if (FAILED(hResult))

			{

				GraphicError (ERRGRAPHICS_CREATE_PRIMARY, ERRGRAPHICS_FILE);

			}
			
			break;

		}

	}

	else 

	{

		GraphicError (ERRGRAPHICS_SET_DISPLAY_MODE, ERRGRAPHICS_FILE);

	}
	
	return (0);

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CScreen::Flip"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			05 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				Flips to the next surface in chain.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CScreen::Flip()

{

	HRESULT hResult;

	hResult = CPrimary.lpSurface->Flip(NULL, DDFLIP_WAIT);

	if (FAILED(hResult))

	{

		GraphicError (ERRGRAPHICS_DD_SET_COOPERATIVE, ERRGRAPHICS_FILE);

	}

	return (0);

}
