//////////////////////////////////////////////////////////////////////
//
//
//		File :					"PhoenixGraphics.h"
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			02 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				DirectDraw object implementation.
//								All primitive graphics functions are
//								declared in this file. 
//								Header file for PhoenixGraphics.h
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
#ifndef		PHOENIXGRAPHICS_H
#define		PHOENIXGRAPHICS_H


//////////////////////////////////////////////////////////////////////
//
//
//		Included headers
//
//
//////////////////////////////////////////////////////////////////////
/*  DirectDraw 7 header file  */
#include	<ddraw.h>


//////////////////////////////////////////////////////////////////////
//
//
//		Definitions
//
//
//////////////////////////////////////////////////////////////////////
#define		SURFACE_VIDEO					1 << 0
#define		SURFACE_SYSTEM					1 << 1

#define		SURFACE_PRIMARY					1 << 0
#define		SURFACE_PRIMARY_SECONDARY		2 << 0

#define		ERRGRAPHICS_NOQUIT				0 << 0
#define		ERRGRAPHICS_QUIT				1 << 0
#define		ERRGRAPHICS_DIALOG				1 << 1
#define		ERRGRAPHICS_FILE				1 << 2

#define		ERRGRAPHICS_CREATE_SURFACE		1 << 0
#define		ERRGRAPHICS_INIT_COLOR			2 << 0
#define		ERRGRAPHICS_LOCK_SURFACE		3 << 0
#define		ERRGRAPHICS_UNLOCK_SURFACE		4 << 0
#define		ERRGRAPHICS_CHECK_LOST			5 << 0
#define		ERRGRAPHICS_RESTORE				6 << 0
#define		ERRGRAPHICS_BLIT				7 << 0
#define		ERRGRAPHICS_BLITEX				8 << 0
#define		ERRGRAPHICS_CLEAR_SURFACE		9 << 0
#define		ERRGRAPHICS_RELEASE_SURFACE		10 << 0
#define		ERRGRAPHICS_DRAW_PIXEL			11 << 0
#define		ERRGRAPHICS_GET_INFO			12 << 0
#define		ERRGRAPHICS_DD_SET_COOPERATIVE	13 << 0
#define		ERRGRAPHICS_DD_CREATE			14 << 0
#define		ERRGRAPHICS_FLIP				15 << 0
#define		ERRGRAPHICS_SET_DISPLAY_MODE	16 << 0
#define		ERRGRAPHICS_CREATE_PRIMARY		17 << 0
#define		ERRGRAPHICS_CREATE_SECONDARY	18 << 0
#define		ERRGRAPHICS_RELEASE_SCREEN		19 << 0


//////////////////////////////////////////////////////////////////////
//
//
//		Classes
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					CSurface
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			02 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Surface object. All the graphics work
//								and tweaking will be done using the
//								CSurface class.
//
//
//		Observations :			Not finished.
//
//
//////////////////////////////////////////////////////////////////////
class	CSurface

{

public:

	LPDIRECTDRAWSURFACE7		lpSurface;
	DDSURFACEDESC2				sSurface;

	WORD						wRedShift;
	WORD						wGreenShift;
	WORD						wBlueShift;
	WORD						wPitch;
	WORD						wWidth;
	WORD						wHeight;
	WORD						*pwBuffer;
	bool						bLocked;
	bool						bSystemSurface;

	/*	Constructors and destructor  */
	CSurface (WORD wWidth, WORD wHeight, BYTE byFlags);
	CSurface ();
	~CSurface ();

	/*	Locking and unlocking surfaces  */
	LockSurface ();
	UnlockSurface ();

	/*	Blit fucntions  */
	SetColorKey (BYTE byType, WORD wRed, WORD wGreen, WORD wBlue);
	Blit (RECT rcArea, RECT rcAreaDst, CSurface CSourceSurface);
	BlitEx (RECT rcArea, CSurface CSourceSurface, DWORD dwFlags);

	/*	Clears surface  */
	ClearSurface (WORD wRed, WORD wGreen, WORD wBlue);

	/*	Draw pixels  */
	DrawPixel (WORD wX, WORD wY, WORD wRed, WORD wGreen, WORD wBlue);
	DrawAlphaPixel (WORD wX, WORD wY, WORD wRed, WORD wGreen, WORD wBlue,
					BYTE byAlpha);

	/*	Draw primitive shapes  */
	DrawLine (RECT rcOutline, WORD wRed, WORD wGreen, WORD wBlue);
	DrawAlphaLine (RECT rcOutline, WORD wRed, WORD wGreen, WORD wBlue, 
				   BYTE byAlpha);

	/*	Load bitmapped images  */
	LoadImage (LPSTR lpstrFileName, DWORD dwType);

	/*	Misc. functions  */
	CheckLost();
	InitColor ();
	GetInfo ();

};


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					CScreen
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			02 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				Screen object. All the display in the 
//								screen will be derived by the actions
//								performed on this class.
//
//
//		Observations :			Not finished.
//
//
//////////////////////////////////////////////////////////////////////
class	CScreen

{

public:

	CSurface					CPrimary;
	CSurface					CSecondary;

	
	/*  Constructor and destructor  */
	CScreen (HWND hWnd);
	~CScreen ();

	/*	Set display mode  */
	SetDisplayMode (WORD wWidth, WORD wHeight, WORD wBpp, BYTE byFlags);

	/*	Flips to the next surface in chain  */
	Flip ();

};

#endif

