//////////////////////////////////////////////////////////////////////
//
//
//		File :					"PhoenixInput.cpp"
//
//
//		Related header file :	"PhoenixInput.h"
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			05 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			05 - 02 - 2000
//
//
//		Purpose :				DirectInput object implementation.
//								All primitive input functions are
//								declared in this file. 
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Included headers
//
//
//////////////////////////////////////////////////////////////////////
/*  Input part of library  */
#include	"PhoenixInput.h"

//////////////////////////////////////////////////////////////////////
//
//
//		Global variables
//
//
//////////////////////////////////////////////////////////////////////
LPDIRECTINPUT7					g_lpDirectInput7;


//////////////////////////////////////////////////////////////////////
//
//
//		Functions
//
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"InputError"
//
//
//		Input :					ULONG ulError - Error that happen
//								WORD Flags	  - Flags
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			04 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			04 - 02 - 2000
//
//
//		Purpose :				Error handler for input fucntions
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
void InputError (ULONG ulError, WORD wFlags)

{

	char		ErrorMsg[256];
	ZeroMemory(ErrorMsg,256);

	switch (ulError)

	{

	case 1:

		strcpy (ErrorMsg, "CreateSurface() failed - Error 0001");

		if (wFlags & ERRINPUT_DIALOG)

		{

			MessageBox (NULL, ErrorMsg, "Error ocurred !", MB_OK | 
						MB_ICONSTOP | MB_APPLMODAL | MB_TOPMOST);

		}

		if (wFlags & ERRINPUT_FILE)

		{

			HANDLE	hErrorLog;
			DWORD	dwBytesWritten;

			hErrorLog = CreateFile ("PhoenixError.log", GENERIC_WRITE, NULL, NULL,
									OPEN_ALWAYS, NULL, NULL);
			WriteFile (hErrorLog, ErrorMsg, sizeof (ErrorMsg), &dwBytesWritten, NULL);
			CloseHandle (hErrorLog);

		}

		break;


	default:

		break;

	}

	if (wFlags & ERRINPUT_QUIT)

	{

		PostQuitMessage (ulError);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CInput::CInput"
//
//
//		Input :					HINSTANCE hInstance - Instance of main
//													  application
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			07 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			07 - 02 - 2000
//
//
//		Purpose :				Creates and initializes the CInput
//								object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CInput::CInput (HINSTANCE hInstance)

{

	HRESULT hResult;

	hResult = DirectInputCreate (hInstance, DIRECTINPUT_VERSION, 
								 (LPDIRECTINPUT*)g_lpDirectInput7, NULL);

	if (FAILED(hResult))

	{

		InputError(ERRINPUT_DI_CREATE, ERRINPUT_QUIT | ERRINPUT_FILE);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CInput::~CInput"
//
//
//		Input :					Nothing
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			07 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			07 - 02 - 2000
//
//
//		Purpose :				Destroys the CInput object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CInput::~CInput ()

{

	HRESULT hResult;

	hResult = g_lpDirectInput7->Release();

	if (FAILED(hResult))

	{

		InputError(ERRINPUT_DI_RELEASE, ERRINPUT_FILE);

	}

}


//////////////////////////////////////////////////////////////////////
//
//
//		Name :					"CInput::CInput"
//
//
//		Input :					HWND hWnd	- Handle to parent window
//								BYTE byType - Type of device
//
//
//		Return :				Nothing
//
//
//		Original author :		Bruno Sousa
//
//
//		Creation date :			07 - 02 - 2000
//
//
//		Last modified by :		Bruno Sousa
//
//
//		Last modified :			07 - 02 - 2000
//
//
//		Purpose :				Creates and initializes the CDevice
//								object.
//
//
//		Observations :			None.
//
//
//////////////////////////////////////////////////////////////////////
CDevice::CDevice (HWND hWnd, BYTE byType)

{

	HRESULT hResult;

	switch (byType)

	{

	case (PKEYBOARD):

		hResult = g_lpDirectInput7->CreateDevice(GUID_SysKeyboard, 
												 &lpDevice, NULL);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_KCREATE_DI_DEVICE, ERRINPUT_FILE);

		}

		hResult = lpDevice->SetCooperativeLevel(hWnd, DISCL_BACKGROUND | 
												DISCL_NONEXCLUSIVE);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_KSET_DI_COOPERATIVE, ERRINPUT_FILE);

		}

		hResult = lpDevice->SetDataFormat(&c_dfDIKeyboard);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_KSET_DATA_FORMAT, ERRINPUT_FILE);

		}

		byType = PKEYBOARD;

		break;

	case (PMOUSE):

		hResult = g_lpDirectInput7->CreateDevice(GUID_SysMouse, 
												 &lpDevice, NULL);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_MCREATE_DI_DEVICE, ERRINPUT_FILE);

		}

		hResult = lpDevice->SetCooperativeLevel(hWnd, DISCL_BACKGROUND | 
												DISCL_NONEXCLUSIVE);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_MSET_DI_COOPERATIVE, ERRINPUT_FILE);

		}

		hResult = lpDevice->SetDataFormat(&c_dfDIMouse);

		if (FAILED(hResult))

		{

			InputError(ERRINPUT_MSET_DATA_FORMAT, ERRINPUT_FILE);

		}

		byType = PMOUSE;

		break;

	}

}